% ################################################## %
% Zeitsimulation und Animation des Einzylindermotors %
% aus den Aufgaben A12, B1, B2                       %
% ################################################## %

% Simulation: Loesung der nichtlinearen Zustandsdifferentialgleichung

% Anfangsbedingenungen
x0 = [5*pi/180; 3*360*pi/180]; % [rad rad/s]
    
% Randbedingungen
tspan = 0:0.0025:1.5;

% Integration der Zustandsgleichung
[t, x] = ode45(@beweggl_funktion,tspan,x0);

% plotte verallgemeinerte Koordinate
figure(1); clf;
gammaV  = x(:,1);    % gamma Integrationsergebnis
DgammaV = x(:,2);    % Dgamma Integrationsergebnis
plot(t,gammaV*180/pi,'b',t,DgammaV*180/pi,'r')
legend({'gamma','Dgamma'})
title('Trajektorien')

% Berechnung der zugehoerigen Lagerreaktionskraefte & Gaskraft
[g_, FG_] = arrayfun(@reaktionskraefte_funktion,gammaV,DgammaV,'UniformOutput',false);
g_V = vertcat(g_{:});
FG_V = vertcat(FG_{:});

% plotte Reaktionskraefte
figure(2);clf;hold on
plot(t,g_V(:,[1 2 4 5 6 8 9]))
plot(t,FG_V,'linewidth',2)
legend({'F1','F2','F4','F5','F6','F8','F9','FG_V'})
title('Reaktionskraefte und Gaskraft')

%% Interpolation und Animation der Ergebnisse
dt            = 0.003; % Zeitschritt
t_interp      = 0:dt:max(t);
gammaV_interp = interp1(t,gammaV,t_interp');
g_V_interp    = interp1(t,g_V,t_interp');
FG_V_interp   = interp1(t,FG_V,t_interp');

% Plot-Werte
LWidth_ = 2;     % Line-Width
JSize   = 8;     % Joint-Size
ASize   = 7;     % Arrow-Size
ADist   = 0.08;  % Arrow-Distance
FScale  = 2.5e-4;% Force-Scale

% Zahlenwerte festlegen
init_vars;

% Anfangszustand
gamma_0 = x0(1);
delta_0 = asin(lambda*sin(gamma_0));

% Positions Vektoren der Gelenke
r1 = [r*cos(gamma_0);
      r*sin(gamma_0);
      0]; 
r2 = [r*cos(gamma_0) + l*cos(delta_0);
      0;
      0];

% Zeichnung des Anfangszustands
h_fig = figure(3); clf; hold on
plot3([0 0 0],[0 r1(2) r2(2)],[0 r1(1) r2(1)],'-','LineWidth',LWidth_)
plot3([0 0 0],[0 r1(2) r2(2)],[0 r1(1) r2(1)],'o','markersize',JSize,...
    'linewidth',LWidth_)
plot3(0,r2(2),r2(1),'s','markersize',60,'LineWidth',LWidth_) % Kolben

% Plot-Einstellungen
axis equal
view([-1 0 0])
yRange = [-r*1.5 r*1.5];
zRange = [-r*1.5 (r+l)*1.3];
ylim(yRange)
zlim(zRange)
zlabel('e_1')
ylabel('e_2')
title('Einzylindermotor')
view([-1 0 0])
pause(0.5)

for idx=1:length(t_interp)
    gamma  = gammaV_interp(idx);
    delta = asin(lambda*sin(gamma));

    % Gelenk-Positionen
    r1 = [r*cos(gamma);
          r*sin(gamma);
          0]; 
    r2 = [r*cos(gamma) + l*cos(delta);
          0;
          0];
    
    % zeichne aktuelle Lage
    figure(h_fig); cla; hold on
    plot3([0 0 0],[0 r1(2) r2(2)],[0 r1(1) r2(1)],'-',...
        'LineWidth',LWidth_)
    plot3([0 0 0],[0 r1(2) r2(2)],[0 r1(1) r2(1)],'o',...
        'markersize',JSize,'linewidth',LWidth_)
    plot3(0,r2(2),r2(1),'s','markersize',60,'LineWidth',LWidth_) % Kolben
    
    % Kraefte an der Kurbel
    if g_V_interp(idx,1)>0          % Kraft e1-Richtung
        plot3([0, 0],[0,0],[FScale*g_V_interp(idx,1)+ADist, ADist],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,0,ADist,'v','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    else                   
        plot3([0,0],[0,0],[FScale*g_V_interp(idx,1)-ADist, -ADist],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,0,-ADist,'^','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    end
    if g_V_interp(idx,2)>0          % Kraft e2-Richtung
        plot3([0, 0],[-FScale*g_V_interp(idx,2)-ADist,-ADist],[0, 0],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,-ADist,0,'<','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    else
        plot3([0, 0],[-FScale*g_V_interp(idx,2)+ADist,ADist],[0, 0],'-',...
            'Color','r','LineWidth',LWidth_);
        plot3(0,ADist,0,'>','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    end

    % Querkraft am Kolben
    if g_V_interp(idx,8)>0          % Kraft e2-Richtung
        plot3([0, 0],[-FScale*g_V_interp(idx,8)-ADist,-ADist],r2(1)+[0, 0],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,-ADist,r2(1)+0,'<','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    else                     
        plot3([0, 0],[-FScale*g_V_interp(idx,8)+ADist,ADist],r2(1)+[0, 0],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,ADist,r2(1)+0,'>','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
     end

    % Kraefte vom Pleuel auf die Kurbel
    if g_V_interp(idx,4)>0          % Kraft e1-Richtung
        plot3([0, 0],r1(2)+[0,0],r1(1)+[-FScale*g_V_interp(idx,4)-ADist,-ADist],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,r1(2)+0,r1(1)-ADist,'^','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    else                     
        plot3([0, 0],r1(2)+[0,0],r1(1)+[-FScale*g_V_interp(idx,4)+ADist,ADist],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,r1(2)+0,r1(1)+ADist,'v','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    end
    if g_V_interp(idx,5)>0          % Kraft e2-Richtung
        plot3([0, 0],r1(2)+[-FScale*g_V_interp(idx,5)-ADist,-ADist],r1(1)+[0, 0],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,r1(2)-ADist,r1(1)+0,'<','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    else                     
        plot3([0, 0],r1(2)+[-FScale*g_V_interp(idx,5)+ADist,ADist],r1(1)+[0, 0],...
            '-','Color','r','LineWidth',LWidth_);
        plot3(0,r1(2)+ADist,r1(1)+0,'>','markersize',ASize,...
            'MarkerEdgeColor','r','MarkerFaceColor','r');
    end 
    
    % Gaskraft
    if FG_V_interp(idx,1)>0          % Kraft e1-Richtung
        plot3([0, 0],[0,0],r2(1)+[10*FScale*FG_V_interp(idx,1)+5*ADist, 5*ADist],...
            '-','Color','g','LineWidth',LWidth_);
        plot3(0,0,r2(1)+5*ADist,'v','markersize',ASize,...
            'MarkerEdgeColor','g','MarkerFaceColor','g');
    end               
   
    % zeige das Bild fuer einen kurzen Moment
    pause(0.01)
    
end